<?php

namespace FiloBlu\Storelocator\Model\Import;

use Magento\ImportExport\Model\Import\ErrorProcessing\ProcessingErrorAggregatorInterface;
use FiloBlu\Storelocator\Model\Import\Stores\RowValidatorInterface as ValidatorInterface;

class Stores extends \Magento\ImportExport\Model\Import\Entity\AbstractEntity {

    const ID = 'store_id';
    const ENABLED = 'enabled';
    const IS_ENABLED_FOR_PICKUP_IN_STORE = 'is_enabled_for_pickup_in_store';
    const TITLE = 'title';
    const LATITUDE = 'latitude';
    const LONGITUDE = 'longitude';
    const STORE_IDS = 'store_ids';
    const STREET = 'street';
    const POSTCODE = 'postcode';
    const CITY = 'city';
    const REGION = 'region';
    const REGION_CODE = 'region_code';
    const COUNTRY = 'country';
    const PHONE_1 = 'phone_1';
    const PHONE_2 = 'phone_2';
    const FAX = 'fax';
    const EMAIL = 'email';
    const WEBSITE_URL = 'website_url';
    const ADDITIONAL_TEXT = 'additional_text';
    // Working times
    const WT_MONDAY = 'wt_monday';
    const WT_TUESDAY = 'wt_tuesday';
    const WT_WEDNESDAY = 'wt_wednesday';
    const WT_THURSDAY = 'wt_thursday';
    const WT_FRIDAY = 'wt_friday';
    const WT_SATURDAY = 'wt_saturday';
    const WT_SUNDAY = 'wt_sunday';
    // Tags
    const TAGS = 'tags';
    // Google My Business
    const GMB_CAT = 'gmb_cat';
    const GMB_LANGUAGE_CODE = 'gmb_language_code';

    protected $_helper;
    protected $logInHistory = true;
    protected $needColumnCheck = false;
    protected $_permanentAttributes = [
        self::ENABLED,
        self::TITLE,
        self::STORE_IDS,
    ];
    protected $validColumnNames = [
        self::ENABLED,
        self::IS_ENABLED_FOR_PICKUP_IN_STORE,
        self::TITLE,
        self::LATITUDE,
        self::LONGITUDE,
        self::STORE_IDS,
        self::STREET,
        self::POSTCODE,
        self::CITY,
        self::REGION,
        self::REGION_CODE,
        self::COUNTRY,
        self::PHONE_1,
        self::PHONE_2,
        self::FAX,
        self::EMAIL,
        self::WEBSITE_URL,
        self::ADDITIONAL_TEXT,
        self::TAGS,
        self::WT_MONDAY,
        self::WT_TUESDAY,
        self::WT_WEDNESDAY,
        self::WT_THURSDAY,
        self::WT_FRIDAY,
        self::WT_SATURDAY,
        self::WT_SUNDAY,
        self::GMB_CAT,
        self::GMB_LANGUAGE_CODE,
    ];

    public function __construct(
    \Magento\Framework\Json\Helper\Data $jsonHelper, \Magento\ImportExport\Helper\Data $importExportData, \Magento\ImportExport\Model\ResourceModel\Import\Data $importData, \Magento\Framework\App\ResourceConnection $resource, \Magento\ImportExport\Model\ResourceModel\Helper $resourceHelper, ProcessingErrorAggregatorInterface $errorAggregator, \FiloBlu\Storelocator\Helper\Import $helper
    ) {
        $this->jsonHelper = $jsonHelper;
        $this->_importExportData = $importExportData;
        $this->_resourceHelper = $resourceHelper;
        $this->_dataSourceModel = $importData;
        $this->_resource = $resource;
        $this->_connection = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        $this->errorAggregator = $errorAggregator;
        $this->_helper = $helper;
    }

    public function getValidColumnNames() {
        return $this->validColumnNames;
    }

    /**
     * Create Advanced price data from raw data.
     *
     * @throws \Exception
     * @return bool Result of operation.
     */
    protected function _importData() {

        if (\Magento\ImportExport\Model\Import::BEHAVIOR_APPEND == $this->getBehavior()) {
            $this->updateData();
        }

        return true;
    }

    /**
     * Entity type code getter.
     *
     * @return string
     */
    public function getEntityTypeCode() {
        return 'filoblustorelocator_import';
    }

    public function validateRow(array $rowData, $rowNum) {

        if (\Magento\ImportExport\Model\Import::BEHAVIOR_APPEND == $this->getBehavior()) {

            // Check enabled
            if ((!isset($rowData[self::ENABLED]) || empty($rowData[self::ENABLED])) && $rowData[self::ENABLED] != 0) {
                $this->addRowError(ValidatorInterface::ERROR_ENABLED_IS_EMPTY, $rowNum);
            } else {
                if (!$this->_helper->validateEnabled($rowData[self::ENABLED])) {
                    $this->addRowError(ValidatorInterface::ERROR_INVALID_ENABLED . " - currentEnabled: " . $rowData[self::ENABLED], $rowNum);
                }
            }

            // Check title
            if (!isset($rowData[self::TITLE]) || empty($rowData[self::TITLE])) {
                $this->addRowError(ValidatorInterface::ERROR_TITLE_IS_EMPTY, $rowNum);
            }

            // Check store ids
            if ((isset($rowData[self::STORE_IDS]) && !empty($rowData[self::STORE_IDS])) && $rowData[self::STORE_IDS] != 0) {
                if (!$this->_helper->validateStoreIds($rowData[self::STORE_IDS])) {
                    $this->addRowError(ValidatorInterface::ERROR_INVALID_STOREID . " - currentStoreIds: " . $rowData[self::STORE_IDS], $rowNum);
                }
            }

            // Check email
            if (isset($rowData[self::EMAIL]) && !empty($rowData[self::EMAIL])) {
                if (!\Zend_Validate::is(trim($rowData[self::EMAIL]), 'EmailAddress')) {
                    $this->addRowError(ValidatorInterface::ERROR_INVALID_EMAIL . " " . trim($rowData[self::EMAIL]), $rowNum);
                }
            }

            // Working Times
            if (isset($rowData[self::WT_MONDAY]) && !empty($rowData[self::WT_MONDAY])) {
                if (!$this->_helper->validateHours($rowData[self::WT_MONDAY])) {
                    $this->addRowError(ValidatorInterface::ERROR_INVALID_HOURS . " - " . self::WT_MONDAY, $rowNum);
                }
            }

            if (isset($rowData[self::WT_TUESDAY]) && !empty($rowData[self::WT_TUESDAY])) {
                if (!$this->_helper->validateHours($rowData[self::WT_TUESDAY])) {
                    $this->addRowError(ValidatorInterface::ERROR_INVALID_HOURS . " - " . self::WT_TUESDAY, $rowNum);
                }
            }
            if (isset($rowData[self::WT_WEDNESDAY]) && !empty($rowData[self::WT_WEDNESDAY])) {
                if (!$this->_helper->validateHours($rowData[self::WT_WEDNESDAY])) {
                    $this->addRowError(ValidatorInterface::ERROR_INVALID_HOURS . " - " . self::WT_WEDNESDAY, $rowNum);
                }
            }
            if (isset($rowData[self::WT_THURSDAY]) && !empty($rowData[self::WT_THURSDAY])) {
                if (!$this->_helper->validateHours($rowData[self::WT_THURSDAY])) {
                    $this->addRowError(ValidatorInterface::ERROR_INVALID_HOURS . " - " . self::WT_THURSDAY, $rowNum);
                }
            }
            if (isset($rowData[self::WT_FRIDAY]) && !empty($rowData[self::WT_FRIDAY])) {
                if (!$this->_helper->validateHours($rowData[self::WT_FRIDAY])) {
                    $this->addRowError(ValidatorInterface::ERROR_INVALID_HOURS . " - " . self::WT_FRIDAY, $rowNum);
                }
            }
            if (isset($rowData[self::WT_SATURDAY]) && !empty($rowData[self::WT_SATURDAY])) {
                if (!$this->_helper->validateHours($rowData[self::WT_SATURDAY])) {
                    $this->addRowError(ValidatorInterface::ERROR_INVALID_HOURS . " - " . self::WT_SATURDAY, $rowNum);
                }
            }
            if (isset($rowData[self::WT_SUNDAY]) && !empty($rowData[self::WT_SUNDAY])) {
                if (!$this->_helper->validateHours($rowData[self::WT_SUNDAY])) {
                    $this->addRowError(ValidatorInterface::ERROR_INVALID_HOURS . " - " . self::WT_SUNDAY, $rowNum);
                }
            }
        }

        return !$this->getErrorAggregator()->isRowInvalid($rowNum);
    }

    protected function updateData() {
        $i = 0;

        while ($bunch = $this->_dataSourceModel->getNextBunch()) {

            $entityList = [];
            foreach ($bunch as $rowNum => $rowData) {

                if ($this->getErrorAggregator()->hasToBeTerminated()) {
                    $this->getErrorAggregator()->addRowToSkip($rowNum);
                    continue;
                }

                $i++;

                foreach ($rowData as $key => $value) {
                    $entityList[$i][$key] = $value;
                }
            }

            $this->countItemsCreated += $i;

            $this->_helper->importDbData($entityList);
        }

        return $this;
    }

}
